close all;
clear all;
clc;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% see https://www.gps.gov/technical/icwg/IS-GPS-200J.pdf for more defintions %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Polynomial: 1 + X3 + X4 + X5 + X6 + X9 + X11 + X13 + X16 + X19 + X21 + X24 + X27
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
P_Mask = [1 0 0 1 0 0 1 0 1 0 0 1 0 0 1 0 1 0 1 0 0 1 1 1 1 0 0]; % X27 --> X0

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Short Cycled Period : 10230
NumIter = 10230;
SV_Index = 4;
L2_CM_Init = {  [7,4,2,4,1,7,6,6,4],... % SV 1
                [7,5,6,0,1,4,0,3,5],... % SV 2
                [0,0,2,7,4,7,1,4,4],... % SV 3
                [0,6,6,2,6,5,7,2,4],... % SV 4 
                [6,0,1,4,0,3,4,7,1],... % SV 5
                [7,0,3,2,3,2,7,3,3],... % SV 6
                [1,2,4,5,1,0,0,7,0],... % SV 7
                [6,1,7,3,1,6,3,6,1],... % SV 8
                [0,4,7,5,4,1,6,2,1],... % SV 9
                [7,3,3,0,3,1,0,4,6],... % SV 10
                [7,1,3,5,1,2,1,4,5],... % SV 11
                [0,2,4,4,3,7,6,0,6],... % SV 12
                [0,2,1,2,6,4,0,0,3],... % SV 13
                [2,3,0,6,5,5,3,5,1],... % SV 14
                [0,0,1,3,1,4,4,0,0],... % SV 15
                [2,2,2,0,2,1,5,0,6],... % SV 16
                [5,4,0,2,6,4,0,2,6],... % SV 17
                [2,0,5,5,2,1,7,0,5],... % SV 18
                [0,6,4,0,2,2,1,4,4],... % SV 19
                [1,2,0,1,6,1,2,7,4],... % SV 20
                [0,4,4,0,2,3,5,3,3],... % SV 21
                [7,2,4,7,4,4,3,2,7],... % SV 22
                [0,4,5,7,4,3,5,7,7],... % SV 23
                [7,4,1,2,0,1,6,6,0],... % SV 24
                [7,0,0,2,7,4,1,3,4],... % SV 25
                [0,1,0,2,4,7,2,6,1],... % SV 26
                [7,1,3,4,3,3,4,4,5],... % SV 27
                [7,3,7,3,2,4,1,6,2],... % SV 28
                [3,1,1,6,2,7,4,3,4],... % SV 29
                [7,1,0,4,5,2,0,0,7],... % SV 30
                [7,2,2,4,6,2,1,3,3],... % SV 31
                [0,5,0,1,7,2,2,1,3]};   % SV 32

L2_CM_End = {   [5,5,2,5,6,6,0,0,2],...
                [0,3,4,4,4,5,0,3,4],...
                [7,2,3,4,4,3,7,1,1],...
                [5,1,1,2,2,2,0,1,3],...
                [4,6,3,0,5,5,2,1,3],...
                [6,6,7,0,4,4,5,2,4],...
                [6,5,2,3,2,2,6,5,3],...
                [5,0,5,7,0,3,3,4,4],...
                [5,2,0,3,0,2,7,7,5],...
                [2,4,4,2,0,5,5,0,6],...
                [2,3,6,1,7,4,0,0,2],...
                [6,5,4,3,0,5,5,3,1],...
                [4,3,5,0,7,0,5,7,1],...
                [6,3,0,4,3,1,2,5,1],...
                [2,3,4,0,4,3,4,1,7],...
                [5,3,5,5,4,0,7,4,5],...
                [0,4,3,0,5,6,7,3,4],...
                [7,3,1,3,0,4,1,0,3],...
                [4,1,2,1,2,0,1,0,5],...
                [3,6,5,6,3,6,1,1,1],...
                [1,4,3,3,2,4,6,5,7],...
                [1,1,0,7,6,6,4,6,2],...
                [6,0,2,4,0,5,2,0,3],...
                [1,7,7,7,3,5,6,5,0],...
                [6,3,0,1,7,7,5,6,0],...
                [6,5,3,4,6,7,1,0,7],...
                [4,0,6,5,7,6,6,3,0],...
                [2,2,1,7,7,7,1,0,0],...
                [7,7,3,2,6,6,6,7,3],...
                [1,0,0,0,1,0,7,1,0],...
                [4,3,1,0,3,7,1,3,2],...
                [6,2,4,1,2,7,4,7,5]};


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Convert Init / End States %
LSFR_Start = zeros(1,27);
LSFR_Stop  = zeros(1,27);
Start = [];
Stop  = [];
for i = 1:9
    Start = [Start,dec2bin(L2_CM_Init{SV_Index}(i),3)];
    Stop  = [Stop,dec2bin(L2_CM_End{SV_Index}(i),3)];
end
for i = 1 :27
    if(Start(i) == '1')
        LSFR_Start(i) = 1;
    end
    if(Stop(i) == '1')
        LSFR_Stop(i) = 1;
    end
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Loop Across All Iterations %
Man = zeros(1,NumIter);
LSFR = LSFR_Start;
for i = 1 :NumIter

    %%%%%%%%%%%%%%%%%
    % Output Sample %
    Res = LSFR(end);

    %%%%%%%%%%%%%%%%%%
    % Register Shift %
    for j = 27:-1:2
        if(P_Mask(j) == 1)
            % Rigth Shift + XOR
            LSFR(j) = mod(LSFR(j-1) + Res,2); 
        else
            % Right Shift
            LSFR(j) = LSFR(j-1);
        end
    end
    LSFR(1) = Res;

    %%%%%%%%%%%%%%%%%%%%%%
    % Save Output Sample %
    Man(i) = Res;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Check if the End State has Been Found %
    if sum(LSFR == LSFR_Stop) == 27
        disp('Gotcha');
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Convert from 0/1 to -1/1?
% Man = Man * 2 - 1;

%%%%%%%%%%%%%%%%
% Save to File %
fid = fopen(['CM_',num2str(SV_Index),'.txt'],'w+');
for i = 1:NumIter
    fprintf(fid,'%d,\n',int32(Man(i)));
end
fclose(fid);













